# -*- coding: utf-8 -*-
#
# namcap tests - tests fro the depends module
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
# 

import os
import unittest
import tempfile
import shutil
import pacman
import Namcap.depends

class DependsTests(unittest.TestCase):
	def setUp(self):
		self.pkginfo = pacman.PacmanPackage({'name': 'package'})

	def test_missing(self):
		self.pkginfo.detected_deps = ["pkg1"]
		e, w, i = Namcap.depends.analyze_depends(self.pkginfo)
		expected_e = [("dependency-detected-not-included %s", "pkg1")]
		self.assertEqual(e, expected_e)
		self.assertEqual(w, [])
		self.assertEqual(i,
				[('depends-by-namcap-sight depends=(%s)', 'pkg1')])

	def test_unneeded(self):
		self.pkginfo["depends"] = ["pkg1"]
		e, w, i = Namcap.depends.analyze_depends(self.pkginfo)
		expected_w = [("dependency-not-needed %s", "pkg1")]
		self.assertEqual(e, [])
		self.assertEqual(w, expected_w)
		self.assertEqual(i,
				[('depends-by-namcap-sight depends=(%s)', '')])

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap tests - anyelf rule
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
# 

import os
from Namcap.tests.makepkg import MakepkgTest
import Namcap.rules.anyelf

class AnyElfTest(MakepkgTest):
	pkgbuild_any = """
pkgname=__namcap_test_anyelf
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=('glibc')
options=(emptydirs)
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/usr/share/directory
  touch ${pkgdir}/usr/share/directory_file
}
"""
	pkgbuild_elf = """
pkgname=__namcap_test_anyelf
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('any')
url="http://www.example.com/"
license=('GPL')
depends=('glibc')
options=(emptydirs)
source=()
build() {
  true
}
package() {
  install -m755 -D /bin/ls ${pkgdir}/usr/bin/ls
}
"""
	def test_not_any_no_elf(self):
		pkgfile = "__namcap_test_anyelf-1.0-1-%(arch)s.pkg.tar" % { "arch": self.arch }
		with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
			f.write(self.pkgbuild_any)
		self.run_makepkg()
		pkg, r = self.run_rule_on_tarball(
				os.path.join(self.tmpdir, pkgfile),
				Namcap.rules.anyelf.package
				)
		self.assertEqual(r.errors, [])
		self.assertEqual(r.warnings, [('no-elffiles-not-any-package', ())])
		self.assertEqual(r.infos, [])

	def test_any_but_elf(self):
		pkgfile = "__namcap_test_anyelf-1.0-1-any.pkg.tar"
		with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
			f.write(self.pkgbuild_elf)
		self.run_makepkg()
		pkg, r = self.run_rule_on_tarball(
				os.path.join(self.tmpdir, pkgfile),
				Namcap.rules.anyelf.package
				)
		self.assertEqual(r.errors, 
				[("elffile-in-any-package %s", "usr/bin/ls")])
		self.assertEqual(r.warnings, [])
		self.assertEqual(r.infos, [])


# vim: set ts=4 sw=4 noet:




# 
# namcap rules - invalidstartdir
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import re
from Namcap.ruleclass import *

class package(PkgbuildRule):
	name = "invalidstartdir"
	description = "Looks for references to $startdir"
	def analyze(self, pkginfo, tar):
		for i in pkginfo.pkgbuild:
			startdirs = re.split('\${?startdir}?"?', i)
			for j in startdirs[1:]:
				if j[:4] != '/pkg' and j[:4] != '/src':
					self.errors.append(("file-referred-in-startdir", ()))
				elif j[:4] == '/pkg':
					self.errors.append(("use-pkgdir", ()))
				elif j[:4] == '/src':
					self.errors.append(("use-srcdir", ()))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - hardlinks
# Copyright (C) 2011 Dan McGee <dan@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

from os.path import dirname
from Namcap.ruleclass import *

class package(TarballRule):
	name = "hardlinks"
	description = "Look for cross-directory/partition hard links"
	def analyze(self, pkginfo, tar):
		hardlinks = [i for i in tar.getmembers() if i.islnk() == True]
		for hardlink in hardlinks:
			if dirname(hardlink.name) != dirname(hardlink.linkname):
				self.errors.append(("cross-dir-hardlink %s %s",
					(hardlink.name, hardlink.linkname)))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - __init__
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

from types import ModuleType
import Namcap.ruleclass

# Tarball rules
from . import (
  anyelf,
  elffiles,
  emptydir,
  fhs,
  filenames,
  fileownership,
  glibfiles,
  gnomemime,
  hardlinks,
  hicoloricons,
  infodirectory,
  kdeprograms,
  libtool,
  licensepkg,
  lotsofdocs,
  mimefiles,
  missingbackups,
  perllocal,
  permissions,
  rpath,
  scrollkeeper,
  shebangdepends,
  sodepends,
  symlink
)

# PKGBUILD and metadata rules
from . import (
  arrays,
  badbackups,
  carch,
  extravars,
  invalidstartdir,
  missingvars,
  pkginfo,
  pkgnameindesc,
  sfurl,
  splitpkgbuild
)

all_rules = {}
for name,value in dict(locals()).items():
	if not isinstance(value, ModuleType):
		continue
	if name == "Namcap.ruleclass":
		continue
	for n, v in value.__dict__.items():
		if (type(v) == type
			and issubclass(v, Namcap.ruleclass.AbstractRule)
			and hasattr(v, "name")):
			all_rules[v.name] = v

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap tests - hicoloricons
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
# 

import os
from Namcap.tests.makepkg import MakepkgTest
import Namcap.depends
import Namcap.rules.hicoloricons

class hicoloriconsTest(MakepkgTest):
	pkgbuild = """
pkgname=__namcap_test_hicoloricons
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=()
source=()
options=(!purge !zipman)
build() {
  true
}
package() {
  mkdir -p "${pkgdir}/usr/share/icons/hicolor/64x64"
}
"""
	def test_hicoloricons_file_exists(self):
		pkgfile = "__namcap_test_hicoloricons-1.0-1-%(arch)s.pkg.tar" % { "arch": self.arch }
		with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
			f.write(self.pkgbuild)
		self.run_makepkg()
		pkg, r = self.run_rule_on_tarball(
				os.path.join(self.tmpdir, pkgfile),
				Namcap.rules.hicoloricons.package
				)
		e, w, i = Namcap.depends.analyze_depends(pkg)
		self.assertEqual(pkg.detected_deps, ["hicolor-icon-theme"])
		self.assertEqual(set(r.errors + e), set([
			('dependency-detected-not-included %s',
				'hicolor-icon-theme'),
			("hicolor-icon-cache-not-updated", ())
		]))
		self.assertEqual(r.warnings + w, [])
		self.assertEqual(r.infos + i, [
			('depends-by-namcap-sight depends=(%s)', 'hicolor-icon-theme')
			])

	valid_pkgbuild = """
pkgname=__namcap_test_hicoloricons
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=('hicolor-icon-theme')
source=()
options=(!purge !zipman)
build() {
  true
}
package() {
  mkdir -p "${pkgdir}/usr/share/icons/hicolor/64x64"
}
"""

	def test_hicoloricons_normal(self):
		pkgfile = "__namcap_test_hicoloricons-1.0-1-%(arch)s.pkg.tar" % { "arch": self.arch }
		with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
			f.write(self.valid_pkgbuild)
		self.run_makepkg()
		pkg, r = self.run_rule_on_tarball(
				os.path.join(self.tmpdir, pkgfile),
				Namcap.rules.hicoloricons.package
				)
		e, w, i = Namcap.depends.analyze_depends(pkg)
		self.assertEqual(pkg.detected_deps, ["hicolor-icon-theme"])
		self.assertEqual(r.errors + e, [
			("hicolor-icon-cache-not-updated", ())
			])
		self.assertEqual(r.warnings + w, [])
		self.assertEqual(r.infos + i, [
			('depends-by-namcap-sight depends=(%s)', 'hicolor-icon-theme')
			])


# vim: set ts=4 sw=4 noet:




# -*- coding: utf-8 -*-
# 
# namcap rules - shebangdepends
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

"""Checks dependencies on programs specified in shebangs."""

import re, os, pacman
import tempfile
import subprocess
from Namcap.util import is_elf, script_type
from Namcap.ruleclass import *

pkgcache = {}

def load(name, path=None):
	if name not in pkgcache:
		pkgcache[name] = pacman.load(name)
	return pkgcache[name]

def scanshebangs(fileobj, filename, scripts):
	"""
	Scan a file for shebang and stores the interpreter name.
	"""

	# test magic bytes
	magic = fileobj.read(2)
	if magic != b"#!":
		return
	# read the rest of file
	tmp = tempfile.NamedTemporaryFile(delete=False)
	tmp.write(magic + fileobj.read())
	tmp.close()

	try:
		cmd = script_type(tmp.name)
		if cmd != None:
			assert(isinstance(cmd, str))
			scripts.setdefault(cmd, {})[filename] = 1
	finally:
		os.unlink(tmp.name)

def findowners(scriptlist):
	pkglist = set()
	scriptfound = set()

	for s in scriptlist:
		p = subprocess.Popen(["which", s],
				stdout = subprocess.PIPE)
		out, _ = p.communicate()
		if p.returncode != 0:
			continue

		# strip leading slash
		scriptpath = out.strip()[1:]

		pacmandb = '/var/lib/pacman/local'
		for i in os.listdir(pacmandb):
			iname = i.split("-")[0]
			if os.path.isfile(pacmandb+'/'+i+'/files'):
				file = open(pacmandb+'/'+i+'/files', 'rb')
				for j in file:
					if scriptpath == j.strip():
						pkglist.add(iname)
						scriptfound.add(s)
				file.close()

	orphans = list(set(scriptlist) - scriptfound)
	return pkglist, orphans

def getprovides(depends, provides):
	for i in depends.keys():
		pac = load(i)

		if pac != None and 'provides' in pac and pac["provides"] != None:
			provides[i] = pac["provides"]

class ShebangDependsRule(TarballRule):
	name = "depends"
	description = "Checks dependencies semi-smartly."
	def analyze(self, pkginfo, tar):
		scriptlist = {}

		for entry in tar:
			if not entry.isfile():
				continue
			f = tar.extractfile(entry)
			scanshebangs(f, entry.name, scriptlist)
			f.close()

		# find packages owning interpreters
		pkglist, orphans = findowners(scriptlist)
		pkginfo.detected_deps.extend(pkglist)

		# Do the script handling stuff
		for i, v in scriptlist.items():
			files = list(v.keys())
			self.infos.append(("script-link-detected %s in %s", (i, str(files))))

		# Handle "no package associated" errors
		self.warnings.extend([("library-no-package-associated %s", i)
			for i in orphans])

		# Check for packages in testing
		if os.path.isdir('/var/lib/pacman/sync/testing'):
			for i in scriptlist:
				p = pacman.load(i, '/var/lib/pacman/sync/testing/')
				q = pacman.load(i)
				if p != None and q != None and p["version"] == q["version"]:
					self.warnings.append(("dependency-is-testing-release %s", i))

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap tests - makepkg launcher
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
# 

import os
import tempfile
import subprocess
import unittest
import shutil
import tarfile
import sys

from ..package import PacmanPackage

makepkg_conf = """
DLAGENTS=('ftp::/usr/bin/wget -c --passive-ftp -t 3 --waitretry=3 -O %%o %%u'
		  'http::/usr/bin/wget -c -t 3 --waitretry=3 -O %%o %%u'
		  'https::/usr/bin/wget -c -t 3 --waitretry=3 --no-check-certificate -O %%o %%u'
		  'rsync::/usr/bin/rsync -z %%u %%o'
		  'scp::/usr/bin/scp -C %%u %%o')
CARCH="%(arch)s"
CHOST="%(arch)s-pc-linux-gnu"
CFLAGS="-march=%(arch)s -mtune=generic -O2 -pipe"
CXXFLAGS="-march=%(arch)s -mtune=generic -O2 -pipe"
LDFLAGS="-Wl,--hash-style=gnu -Wl,--as-needed"
BUILDENV=(fakeroot !distcc color !ccache)
OPTIONS=(strip docs libtool emptydirs zipman purge)
INTEGRITY_CHECK=(md5)
STRIP_BINARIES="--strip-all"
STRIP_SHARED="--strip-unneeded"
STRIP_STATIC="--strip-debug"
MAN_DIRS=({usr{,/local}{,/share},opt/*}/{man,info})
DOC_DIRS=(usr/{,local/}{,share/}{doc,gtk-doc} opt/*/{doc,gtk-doc})
STRIP_DIRS=(bin lib sbin usr/{bin,lib,sbin,local/{bin,lib,sbin}} opt/*/{bin,lib,sbin})
PURGE_TARGETS=(usr/{,share}/info/dir .packlist *.pod)
PKGEXT='.pkg.tar.xz'
SRCEXT='.src.tar.gz'
"""

class MakepkgTest(unittest.TestCase):
	def setUp(self):
		self.tmpdir = tempfile.mkdtemp()
		self.arch = subprocess.getoutput("arch")
		with open(os.path.join(self.tmpdir, "makepkg.conf"), "w") as f:
			f.write(makepkg_conf % {"arch": self.arch})

	def tearDown(self):
		shutil.rmtree(self.tmpdir)

	def run_makepkg(self):
		pwd = os.getcwd()
		os.chdir(self.tmpdir)
		p = subprocess.Popen(["makepkg", "-f", "-d"],
				env = { "MAKEPKG_CONF": "./makepkg.conf" },
				stdout = subprocess.PIPE, stderr = subprocess.PIPE)
		out1, out2 = p.communicate()
		if p.returncode != 0:
			sys.stderr.buffer.write(out1)
			sys.stderr.buffer.write(out2)
			raise RuntimeError("makepkg returned an error")
		os.chdir(pwd)

	def run_rule_on_tarball(self, filename, rule):
		ret = subprocess.call(["unxz", filename + ".xz"])
		self.assertEqual(ret, 0)

		# process PKGINFO
		tar = tarfile.open(filename)
		info = tar.extractfile('.PKGINFO')
		pkg = PacmanPackage(pkginfo =
				info.read().decode("utf-8", "ignore"))
		info.close()
		tar.close()

		tar = tarfile.open(filename)
		r = rule()
		r.analyze(pkg, tar)
		tar.close()
		return pkg, r

# vim: set ts=4 sw=4 noet:




# -*- coding: utf-8 -*-
# 
# namcap rules - package info structure
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import re
import collections

DEPENDS_RE = re.compile("([^<>=:]+)([<>]?=.*)?(: .*)?")

def strip_depend_info(value):
	"""
	Strip all the depend version info off ('neon>=0.25.5-4' => 'neon').
	Also remove any trailing description (for optdepends)
	"""
	m = DEPENDS_RE.match(value)
	if m is None:
		raise ValueError("Invalid dependency specification")
	return m.group(1)

class PacmanPackage(collections.MutableMapping):
	strings = ['base', 'name', 'version', 'desc', 'url', 'builddate',
			'packager', 'install', 'filename', 'csize', 'isize',
			'pkgfunction']
	equiv_vars = {
		'pkgname': 'name',
		'md5sum': 'md5sums',
		'sha1sum': 'sha1sums',
		'depend': 'depends',
		'pkgdesc': 'desc',
		'size': 'isize',
		'optdepend': 'optdepends',
		}

	@classmethod
	def canonical_varname(cls, varname):
		try:
			return cls.equiv_vars[varname]
		except KeyError:
			return varname

	def __init__(self, data = None, pkginfo = None, db = None):
		"""
		A PacmanPackage object can be ininitialised from:
		* a dictionary (then its contents are updated accordingly)
		* the contents of a .PKGINFO file
		* the contents of a database entry or the output of parsepkgbuild
		"""

		# Usual attributes
		self.is_split = False
		self.detected_deps = []
		self._data = {}

		# Init from a dictionary
		if isinstance(data, dict):
			for k, v in data.items():
				self[k] = v

		# Parsing of .PKGINFO files from tarballs
		if isinstance(pkginfo, str):
			for i in pkginfo.splitlines():
				m = re.match('(.*) = (.*)', i)
				if m != None:
					lhs = m.group(1)
					rhs = m.group(2)
					if rhs != '':
						self.setdefault(lhs, []).append(rhs)
		elif pkginfo is not None:
			raise TypeError("argument 'pkginfo' must be a string")

		# Parsing of database entries or parsepkgbuild output
		if isinstance(db, str):
			attrname = None
			if '\0' in db:
				self.is_split = True
				parts = db.split("\0")
				self.subpackages = [PacmanPackage(db = s) for s in parts[1:]]
				db = parts[0]
			for line in db.split('\n'):
				if line.startswith('%'):
					attrname = line.strip('%').lower()
				elif line.strip() != '':
					self.setdefault(attrname, []).append(line)
		elif db is not None:
			raise TypeError("argument 'pkginfo' must be a string")

		# Cleanup
		self.process()

	def __iter__(self):
		return iter(self._data)

	def __len__(self):
		return len(self._data)

	def __getitem__(self, key):
		return self._data[self.canonical_varname(key)]

	def __setitem__(self, key, value):
		k = self.canonical_varname(key)
		self._data[k] = value

	def __contains__(self, key):
		return self.canonical_varname(key) in self._data

	def __delitem__(self, key):
		del self._data[self.canonical_varname(key)]

	def process_strings(self):
		"""
		Turn all the instance properties listed in self.strings into strings instead of lists
		"""
		for i in self.strings:
			if i in self:
				if isinstance(self[i], list):
					self[i] = self[i][0]

	def clean_depends(self):
		"""
		Strip all the depend version info off ('neon>=0.25.5-4' => 'neon').
		Also clean our optdepends and remove any trailing description.
		The original arrays are copied to orig_depends and orig_optdepends respectively.
		"""

		if 'depends' in self._data:
			self["orig_depends"] = self["depends"]
			self["depends"] = [strip_depend_info(d) for d in self['orig_depends']]
		if 'makedepends' in self._data:
			self["orig_makedepends"] = self["makedepends"]
			self["makedepends"] = [strip_depend_info(d) for d in self['orig_makedepends']]
		if 'optdepends' in self._data:
			self["orig_optdepends"] = self["optdepends"]
			self["optdepends"] = [strip_depend_info(d) for d in self['orig_optdepends']]
		if 'provides' in self._data:
			self["orig_provides"] = self["provides"]
			self["provides"] = [strip_depend_info(d) for d in self['orig_provides']]

	def process(self):
		"""
		After all the text processing happens, call this to sanitize the PacmanPackage object a bit
		"""
		self.process_strings()
		self.clean_depends()

	def __repr__(self):
		if not self.is_split:
			return "PacmanPackage(%s)" % repr(self._data)
		else:
			children = ','.join(repr(p) for p in self.subpackages)
			return ("PacmanPackage(%s,[%s])" %
					(repr(self._data), children))

# vim: set ts=4 sw=4 noet:



import os
import unittest
import tempfile
import shutil

import pacman

pkgbuild = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=('glibc' 'foobar')
optdepends=('libabc: provides the abc feature')
provides=('yourpackage>=0.9')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""

class PkgbuildLoaderTests(unittest.TestCase):
	def setUp(self):
		self.tmpdir = tempfile.mkdtemp()
		tmpname = os.path.join(self.tmpdir, "PKGBUILD")
		with open(tmpname, 'w') as f:
			f.write(pkgbuild)
		self.pkginfo = pacman.load(tmpname)

	def tearDown(self):
		shutil.rmtree(self.tmpdir)

	def test_pkgname(self):
		self.assertEqual(self.pkginfo['name'], "mypackage")
		self.assertEqual(self.pkginfo['pkgname'], "mypackage")
	def test_version(self):
		self.assertEqual(self.pkginfo['version'], "1.0-1")
	def test_pkgdesc(self):
		self.assertEqual(self.pkginfo['desc'], "A package")
	def test_url(self):
		self.assertEqual(self.pkginfo['url'], "http://www.example.com/")

	def test_depends(self):
		self.assertEqual(self.pkginfo['depends'], ["glibc", "foobar"])
		self.assertEqual(self.pkginfo['orig_depends'],
				["glibc", "foobar"])
	def test_optdeps(self):
		self.assertEqual(self.pkginfo['optdepends'], ["libabc"])
		self.assertEqual(self.pkginfo['orig_optdepends'],
				["libabc: provides the abc feature"])
	def test_provides(self):
		self.assertEqual(self.pkginfo['provides'], ["yourpackage"])
		self.assertEqual(self.pkginfo['orig_provides'],
				["yourpackage>=0.9"])

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap tests - non ascii filenames
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
# 

import os
from Namcap.tests.makepkg import MakepkgTest
from Namcap.rules import fhs

class FHSTest(MakepkgTest):
	pkgbuild = """
pkgname=__namcap_test_nonfhs
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/weird/directory
  touch ${pkgdir}/weird/directory/file
}
"""
	def test_nonfhs(self):
		"Package with files in /weird"
		pkgfile = "__namcap_test_nonfhs-1.0-1-%(arch)s.pkg.tar" % { "arch": self.arch }
		with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
			f.write(self.pkgbuild)
		self.run_makepkg()
		pkg, r = self.run_rule_on_tarball(
				os.path.join(self.tmpdir, pkgfile),
				fhs.FHSRule
				)
		self.assertEqual(r.errors, [])
		self.assertEqual(set(r.warnings), set([
			("file-in-non-standard-dir %s", "weird"),
			("file-in-non-standard-dir %s", "weird/directory"),
			("file-in-non-standard-dir %s", "weird/directory/file")
			]))
		self.assertEqual(r.infos, [])

	pkgbuild_man = """
pkgname=__namcap_test_nonfhs
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/usr/man
  touch ${pkgdir}/usr/man/something.1
}
"""
	def test_nonfhsman(self):
		"Package with man pages in /usr/man"
		pkgfile = "__namcap_test_nonfhs-1.0-1-%(arch)s.pkg.tar" % { "arch": self.arch }
		with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
			f.write(self.pkgbuild_man)
		self.run_makepkg()
		pkg, r = self.run_rule_on_tarball(
				os.path.join(self.tmpdir, pkgfile),
				fhs.FHSManpagesRule
				)
		self.assertEqual(r.errors,
				[("non-fhs-man-page %s", "usr/man/something.1.gz")])
		self.assertEqual(r.warnings, [])
		self.assertEqual(r.infos, [])

	pkgbuild_info = """
pkgname=__namcap_test_nonfhs
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=('glibc')
source=()
build() {
  true
}
package() {
  mkdir -p ${pkgdir}/usr/info
  touch ${pkgdir}/usr/info/something
}
"""
	def test_nonfhsinfo(self):
		"Package with info pages in /usr/info"
		pkgfile = "__namcap_test_nonfhs-1.0-1-%(arch)s.pkg.tar" % { "arch": self.arch }
		with open(os.path.join(self.tmpdir, "PKGBUILD"), "w") as f:
			f.write(self.pkgbuild_info)
		self.run_makepkg()
		pkg, r = self.run_rule_on_tarball(
				os.path.join(self.tmpdir, pkgfile),
				fhs.FHSInfoPagesRule
				)
		self.assertEqual(r.errors,
				[("non-fhs-info-page %s", "usr/info/something.gz")])
		self.assertEqual(r.warnings, [])
		self.assertEqual(r.infos, [])



# vim: set ts=4 sw=4 noet:




# -*- coding: utf-8 -*-
#
# namcap rules - missingvars
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

"""Checks for missing variables in PKGBUILD"""

import re
from Namcap.ruleclass import *

RE_IS_HEXNUMBER = re.compile("[0-9a-f]+")

class ChecksumsRule(PkgbuildRule):
	name = "checksums"
	description = "Verifies checksums are included in a PKGBUILD"
	def analyze(self, pkginfo, tar):
		checksums=[('md5', 32), ('sha1', 40), ('sha256', 64), ('sha384', 96), ('sha512', 128)]

		if "source" in pkginfo:
			haschecksums = False
			for sumname, sumlen in checksums:
				if (sumname + 'sums') in pkginfo:
					haschecksums = True
			if not haschecksums:
				self.errors.append(("missing-checksums", ()))
		else:
			pkginfo["source"] = []

		for sumname, sumlen in checksums:
			sumname += 'sums'
			if sumname in pkginfo:
				if len(pkginfo["source"]) > len(pkginfo[sumname]):
					self.errors.append(("not-enough-checksums %s %i needed",
						          (sumname, len(pkginfo["source"]))))
				elif len(pkginfo["source"]) < len(pkginfo[sumname]):
					self.errors.append(("too-many-checksums %s %i needed",
						          (sumname, len(pkginfo["source"]))))
				for sum in pkginfo[sumname]:
					if len(sum) != sumlen or not RE_IS_HEXNUMBER.match(sum):
						self.errors.append(("improper-checksum %s %s", (sumname, sum)))

class TagsRule(PkgbuildRule):
	name = "tags"
	description = "Looks for Maintainer and Contributor comments"
	def analyze(self, pkginfo, tar):
		contributortag = 0
		maintainertag = 0
		idtag = 0
		for i in pkginfo.pkgbuild:
			if re.match("#\s*Contributor\s*:", i) != None:
				contributortag = 1
			if re.match("#\s*Maintainer\s*:", i) != None:
				maintainertag = 1

		if contributortag != 1:
			self.infos.append(("missing-contributor", ()))

		if maintainertag != 1:
			self.warnings.append(("missing-maintainer", ()))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - emptydir
# Copyright (C) 2004-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import os
from Namcap.ruleclass import *

class package(TarballRule):
	name = "emptydir"
	description = "Warns about empty directories in a package"
	def analyze(self, pkginfo, tar):
		dirs = []
		entries = []
		for entry in tar:
			if entry.isdir():
				dirs.append(entry.name.rstrip("/"))
			entries.append(entry.name.rstrip("/"))
		nonemptydirs = [os.path.dirname(x) for x in entries]
		self.warnings = [("empty-directory %s", d)
				for d in (set(dirs) - set(nonemptydirs))]

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - permissions
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import stat
from Namcap.ruleclass import *

class package(TarballRule):
	name = "permissions"
	description = "Checks file permissions."
	def analyze(self, pkginfo, tar):
		for i in tar.getmembers():
			if not i.mode & stat.S_IROTH and not (i.issym() or i.islnk()):
				self.warnings.append(("file-not-world-readable %s", i.name))
			if i.mode & stat.S_IWOTH and not (i.issym() or i.islnk()):
				self.warnings.append(("file-world-writable %s", i.name))
			if not i.mode & stat.S_IXOTH and i.isdir():
				self.warnings.append(("directory-not-world-executable %s", i.name))
			if str(i.name).endswith('.a') and i.mode != 0o644 and i.mode != 0o444:
				self.warnings.append(("incorrect-library-permissions %s", i.name))
			if i.mode & (stat.S_ISUID | stat.S_ISGID):
				self.warnings.append(("file-setugid %s", i.name))

# vim: set ts=4 sw=4 noet:



#
# namcap rules - hicoloricons
# Copyright (C) 2009 Abhishek Dasgupta <abhidg@gmail.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from Namcap.ruleclass import *

class package(TarballRule):
	name = "hicoloricons"
	description = "Checks whether the hicolor icon cache is updated."
	def analyze(self, pkginfo, tar):
		if "usr/share/icons/hicolor" in tar.getnames():
			pkginfo.detected_deps.append("hicolor-icon-theme")

			if ".INSTALL" not in tar.getnames():
				self.errors.append(("hicolor-icon-cache-not-updated", ()))
			else:
				f = tar.extractfile(".INSTALL")
				install_script = f.read().decode("utf-8", "ignore")
				if ("gtk-update-icon-cache" not in install_script) and ("xdg-icon-resource" not in install_script):
					self.errors.append(("hicolor-icon-cache-not-updated", ()))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - lotsofdocs
# Copyright (C) 2009 Dan McGee <dan@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

from Namcap.ruleclass import *

class package(TarballRule):
	name = "lots-of-docs"
	description = "See if a package is carrying more documentation than it should"
	def analyze(self, pkginfo, tar):
		if ('name' in pkginfo and
				(pkginfo["name"].endswith('-doc') or
				 pkginfo["name"].endswith('-docs'))):
			# Don't do anything if the package is called "*-doc"
			return
		docdir = 'usr/share/doc'
		size = 0
		docsize = 0

		for i in tar.getmembers():
			if i.name.startswith(docdir):
				docsize += i.size
			size += i.size

		if size > 0:
			ratio = docsize / float(size)
			if ratio > 0.50:
				self.warnings.append(("lots-of-docs %f", ratio * 100))

# vim: set ts=4 sw=4 noet:



#
# namcap rules - anyelf
# Copyright (C) 2010 Dan McGee <dan@archlinux.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Check for ELF files to see if a package should be 'any' architecture
"""

import os
from Namcap.util import is_elf, clean_filename
from Namcap.ruleclass import *

class package(TarballRule):
	name = "anyelf"
	description = "Check for ELF files to see if a package should be 'any' architecture"
	def analyze(self, pkginfo, tar):
		found_elffiles = []

		for entry in tar.getmembers():
			if not entry.isfile():
				continue
			f = tar.extractfile(entry)
			if f.read(4) == b"\x7fELF":
				found_elffiles.append(entry.name)
			f.close()

		if pkginfo["arch"] and pkginfo["arch"][0] == 'any':
			self.errors = [("elffile-in-any-package %s", i)
					for i in found_elffiles]
		else:
			if len(found_elffiles) == 0:
				self.warnings.append(("no-elffiles-not-any-package", ()))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - split package checks
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

"This module contains rules that check coherence of split PKGBUILDs."

from Namcap.ruleclass import PkgbuildRule
import Namcap.depends

class PackageFunctionsRule(PkgbuildRule):
	"""
	This rule checks for existence of package_* functions
	in split PKGBUILDs. It relies on the output of parsepkgbuild
	which gives the type of the package_xxx object in the PKGBUILD.
	"""
	name = "splitpkgfunctions"
	description = "Checks that all package_* functions exist."

	def analyze(self, pkginfo, pkgbuild):
		if not pkginfo.is_split:
			return
		# All subpackages should have a valid package function
		for subpkg in pkginfo.subpackages:
			if subpkg["pkgfunction"] != "function":
				self.errors.append(
						("missing-pkgfunction %s", subpkg["name"])
						)

class SplitPkgMakedepsRule(PkgbuildRule):
	"""
	This rule checks that the global makedepends of a split
	PKGBUILD cover all needed depends of individual children
	packages.
	"""
	name = "splitpkgmakedeps"
	description = "Checks that a split PKGBUILD has enough makedeps."

	def analyze(self, pkginfo, pkgbuild):
		if not pkginfo.is_split:
			return
		# Find dependencies specified in makedepends
		global_deps = set()
		global_deps.update(pkginfo["names"])
		if "depends" in pkginfo:
			global_deps.update(pkginfo["depends"])
		if "makedepends" in pkginfo:
			global_deps.update(pkginfo["makedepends"])

		global_deps |= Namcap.depends.getcovered(global_deps)

		# Read dependencies specified in subpackages
		local_deps = set()
		for s in pkginfo.subpackages:
			if "depends" in s:
				local_deps.update(s["depends"])
			if "makedepends" in s:
				local_deps.update(s["makedepends"])

		if not local_deps.issubset(global_deps):
			missing = list(local_deps - global_deps)
			self.errors.append(("missing-makedeps %s", str(missing)))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - array
# Copyright (C) 2003-2009 Jesse Young <jesseyoung@gmail.com>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

"""Verifies that array variables are actually arrays"""

import re
from Namcap.ruleclass import *

class package(PkgbuildRule):
	name = "array"
	description = "Verifies that array variables are actually arrays"
	def analyze(self, pkginfo, tar):
		arrayvars = ['arch', 'license', 'depends', 'makedepends',
			 'optdepends', 'provides', 'conflicts' , 'replaces',
			 'backup', 'source', 'noextract', 'md5sums',
			 'sha1sums', 'sha256sums', 'sha384sums', 'sha512sums']
		for i in pkginfo.pkgbuild:
			m = re.match('\s*(.*)\s*=\s*(.*)$', i)
			for j in arrayvars:
				if m and m.group(1) == j:
					if not m.group(2).startswith('('):
						self.warnings.append(("variable-not-array %s", j))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - pacman package interface
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import tarfile, os, re, subprocess
import sys
from Namcap.package import PacmanPackage

pacmandb = '/var/lib/pacman/local/'

for i in open('/etc/pacman.conf'):
	m = re.match('\s*DBPath\s*=\s*([\S^#]+)', i)
	if m != None:
		pacmandb = os.path.join(m.group(1), 'local/')
		break

def load(package, root=None):
	if root == None:
		root = pacmandb
	# We know it's a local package
	if os.path.isfile(package) and tarfile.is_tarfile(package):
		pkgtar = tarfile.open(package, "r")
		if not pkgtar:
			return None
		if not '.PKGINFO' in pkgtar.getnames():
			return None
		info_f = pkgtar.extractfile('.PKGINFO')
		info = info_f.read().decode("utf-8", "ignore")
		info_f.close()
		ret = PacmanPackage(pkginfo = info)
		pkgtar.close()
		return ret

	# Ooooo, it's a PKGBUILD
	elif package.endswith('PKGBUILD'):
		# Load all the data like we normally would
		workingdir = os.path.dirname(package)
		if workingdir == '':
			workingdir = None
		filename = os.path.basename(package)
		process = subprocess.Popen(['parsepkgbuild', filename],
				stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=workingdir)
		data = process.communicate()
		# this means parsepkgbuild returned an error, so we are not valid
		if process.returncode > 0:
			if data[0]:
				print("Error:", data[0].decode("utf-8", "ignore"))
			if data[1]:
				print("Error:", data[1].decode("utf-8", "ignore"), file=sys.stdout)
			return None
		ret = PacmanPackage(db = data[0].decode('utf-8', 'ignore'))

		# Add a nice little .pkgbuild surprise
		pkgbuild = open(package, errors="ignore")
		ret.pkgbuild = pkgbuild.read().replace("\\\n", " ").splitlines()
		pkgbuild.close()

		return ret

	else:
		searchstr = re.compile('(.*)-([^-]*)-([^-]*)')
		for i in os.listdir(root):
			n = searchstr.match(i)
			if n == None:
				continue
			if n.group(1) == package:
				# We found the package!
				return loadfromdir(os.path.join(root, i))

		# Maybe it's a provides then...
		for i in os.listdir(root):
			prov = loadfromdir(os.path.join(root, i))

			if prov != None and 'provides' in prov and package in prov["provides"]:
				return prov

		return None

def loadfromdir(directory):
	"""Read package information from a directory in the database"""
	if not os.path.isdir(directory):
		return None

	data = ""
	for info in ('desc', 'depends', 'files'):
		path = os.path.join(directory, info)
		if os.path.isfile(path):
			dbfile = open(path, encoding="utf-8", errors="ignore")
			data += dbfile.read()
			dbfile.close()

	ret = PacmanPackage(db = data)
	return ret

def getprovides(provides):
	packagelist = []

	searchstr = re.compile('(.*)-([^-]*)-([^-]*)')
	for i in os.listdir(pacmandb):
		pac = loadfromdir(os.path.join(pacmandb, i))
		if 'provides' in pac and provides in pac["provides"]:
			packagelist.append(pac.name)

	return packagelist
# vim: set ts=4 sw=4 noet:



# 
# namcap rules - mimefiles
# Copyright (C) 2009 Hugo Doria <hugo@archlinux.org>
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
# 

from Namcap.ruleclass import *

class MimeInfoRule(TarballRule):
	name = "mimefiles"
	description = "Check for files in /usr/share/mime"
	def analyze(self, pkginfo, tar):
		if 'usr/share/mime' in tar.getnames():
			pkginfo.detected_deps.append("shared-mime-info")
			if ".INSTALL" not in tar.getnames():
				self.errors.append(("mime-cache-not-updated", ()))
			else:
				f = tar.extractfile(".INSTALL")
				if b"update-mime-database" not in f.read():
					self.errors.append(("mime-cache-not-updated", ()))

class MimeDesktopRule(TarballRule):
	name = "mimedesktop"
	description = "Check that MIME associations are updated"
	def analyze(self, pkginfo, tar):
		desktop_db_updated = False
		has_mime_desktop = False
		for entry in tar:
			if (entry.name.startswith("usr/share/applications")
					and entry.name.endswith(".desktop")):
				f = tar.extractfile(entry)
				for l in f:
					if l.startswith(b"MimeType="):
						has_mime_desktop = True
						break
				f.close()
			if entry.name == ".INSTALL":
				f = tar.extractfile(entry)
				if b"update-desktop-database" in f.read():
					desktop_db_updated = True
				f.close()

		if has_mime_desktop:
			pkginfo.detected_deps.append("desktop-file-utils")
			if not desktop_db_updated:
				self.errors.append(("desktop-database-not-updated", ()))

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap - Class hierarchy for rules
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# 

"""
This module defines the base classes from which Namcap rules are derived
and how they are meant to be used.
"""

# pacman 3 does not need classes to derive from object
class AbstractRule(object):
	"The parent class of all rules"
	def __init__(self):
		self.errors = []
		self.warnings = []
		self.infos = []

class PkgInfoRule(AbstractRule):
	"The parent class of rules that process package metadata"
	pass

class PkgbuildRule(AbstractRule):
	"The parent class of rules that process PKGBUILDs"
	pass

class TarballRule(AbstractRule):
	"The parent class of rules that process tarballs"
	pass

class PkgdirRule(AbstractRule):
	"""The parent class of rules that process pkgdirs
	(extracted tarballs)"""
	pass

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - carch
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

"""Verifies that no specific host type is used"""

import re
from Namcap.ruleclass import *

class package(PkgbuildRule):
	name = "carch"
	description = "Verifies that no specific host type is used"
	def analyze(self, pkginfo, tar):
		arches = ['i686', 'i586', 'x86_64']
		archmatch = re.compile('%s' % '|'.join(arches))
		# Match an arch=(i686) line
		archline = re.compile('arch=\w*(%s)' % '|'.join(arches))
		# Match a [ "$CARCH" = "x86_64" ] line
		archif = re.compile('''\[\s*("|')\$CARCH("|').*("|')(%s)("|')\s*\]''' % '|'.join(arches))
		for i in pkginfo.pkgbuild:
			if archmatch.match(i):
				if not archline.match(i) and not archif.match(i):
					self.warnings.append(("specific-host-type-used %s", ",".join(arches)))

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
# 
# namcap rules - depends
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

"""Checks dependencies semi-smartly."""

import re, os, os.path, pacman
import subprocess
import tempfile
from Namcap.util import is_elf, script_type
from Namcap.ruleclass import *

pkgcache = {}

def load(name, path=None):
	global pkgcache
	if name not in pkgcache:
		pkgcache[name] = pacman.load(name)
	return pkgcache[name]

def getcovered(dependlist, covereddepend = None):
	"""
	Fills covereddepend with the full dependency tree
	of dependlist (iterable of package names)
	"""
	if covereddepend is None:
		covereddepend = set()

	for i in dependlist:
		pac = load(i)
		if pac != None and "depends" in pac:
			newdeps = [j for j in pac["depends"]
					if j != None and j not in covereddepend]
			covereddepend.update(newdeps)
			getcovered(newdeps, covereddepend)

	return covereddepend

def getprovides(depends, provides):
	for i in depends:
		pac = load(i)

		if pac != None and "provides" in pac and pac["provides"] != None:
			provides[i] = pac["provides"]

def analyze_depends(pkginfo):
	errors, warnings, infos = [], [], []

	# compute needed dependencies + recursive
	dependlist = pkginfo.detected_deps
	covereddepend = getcovered(dependlist)
	for i in covereddepend:
		infos.append(("dependency-covered-by-link-dependence %s", i))

	# Find all the covered dependencies from the PKGBUILD
	pkginfo.setdefault("depends", [])
	pkgdepend = set(pkginfo["depends"])
	# Include the optdepends from the PKGBUILD
	pkginfo.setdefault("optdepends", [])
	pkgdepend |= set(pkginfo["optdepends"])

	pkgcovered = getcovered(pkgdepend)

	# Our detected dependencies not covered by PKGBUILD
	smartdepend = set(dependlist) - covereddepend

	# Get the provides so we can reference them later
	smartprovides = {}
	getprovides(dependlist, smartprovides)

	# The set of all provides for detected dependencies
	allprovides = set()
	for plist in smartprovides.values():
		allprovides |= set(plist)

	# Do the actual message outputting stuff
	for i in smartdepend:
		# If (i is not in the PKGBUILD's dependencies
		# and i isn't the package name
		# and (if (there are provides for i) then
		#      (those provides aren't included in the package's dependencies))
		# )
		all_dependencies = pkginfo['depends'] + pkginfo['optdepends'] + list(pkgcovered)
		if (i not in all_dependencies and i != pkginfo["name"]
				and ((i not in smartprovides)
					or len([c for c in smartprovides[i] if c in pkgcovered]) == 0)):
			errors.append(("dependency-detected-not-included %s", i))

	for i in pkginfo["depends"]:
		if i in covereddepend and i in dependlist:
			warnings.append(("dependency-already-satisfied %s", i))
		# if i is not in the depends as we see them
		# and it's not in any of the provides from said depends
		elif i not in smartdepend and i not in allprovides:
			warnings.append(("dependency-not-needed %s", i))
	infos.append(("depends-by-namcap-sight depends=(%s)", ' '.join(smartdepend) ))

	return errors, warnings, infos

# vim: set ts=4 sw=4 noet:



import os
import unittest

def getTestSuite():
	loader = unittest.TestLoader()
	return loader.discover(start_dir = os.path.dirname(__file__))

if __name__ == "__main__":
	runner = unittest.TextTestRunner()
	runner.run(getTestSuite())

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap tests - pkgbuild rule testing template
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
# 

import os
import unittest
import tempfile
import shutil

import pacman
import Namcap.ruleclass

class PkgbuildTest(unittest.TestCase):
	"""
	This class is the template for unit tests concerning PKGBUILD rules.
	The setUp()	method should set self.rule to a class derived from
	PkgbuildRule.

	The usual way to do that is by defining

	def preSetUp(self):
		self.rule = myRuleClass
		# Add standard valid PKGBUILDs to tests
		self.test_valid = PkgbuildTest.valid_tests
	"""

	def preSetUp(self):
		pass

	def setUp(self):
		self.preSetUp()
		self.tmpdir = tempfile.mkdtemp()
		self.tmpname = os.path.join(self.tmpdir, "PKGBUILD")

	def run_on_pkg(self, p):
		with open(self.tmpname, 'w') as f:
			f.write(p)
		pkginfo = pacman.load(self.tmpname)
		r = self.rule()
		if isinstance(r, Namcap.ruleclass.PkgInfoRule):
			if pkginfo.is_split:
				for p in pkginfo.subpackages:
					r.analyze(p, self.tmpname)
			else:
				r.analyze(pkginfo, self.tmpname)
		if isinstance(r, Namcap.ruleclass.PkgbuildRule):
			r.analyze(pkginfo, self.tmpname)
		return r

	def valid_tests(self):
		"Valid PKGBUILDS"
		for p in valid_pkgbuilds:
			r = self.run_on_pkg(p)
			self.assertEqual(r.errors, [])
			self.assertEqual(r.warnings, [])
			self.assertEqual(r.errors, [])

	def tearDown(self):
		shutil.rmtree(self.tmpdir)

_basic = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=('glibc' 'pacman')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  make DESTDIR="${pkgdir}" install
}
"""

_split = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgbase=mysplitpackage
pkgname=('mypackage1' 'mypackage2' 'mypackage3')
pkgver=1.0
pkgrel=1
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
makedepends=('make' 'gtk2')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgbase}-${pkgver}
  ./configure --prefix=/usr
  make
}

package_mypackage1() {
  pkgdesc="Package 1"
  depends=("gtk2")
  cd "${srcdir}"/${pkgbase}-${pkgver}
  make DESTDIR="${pkgdir}" install1
}

package_mypackage2() {
  pkgdesc="Package 2"
  depends=("glib2" "mypackage1")
  cd "${srcdir}"/${pkgbase}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install2
}

package_mypackage3() {
  pkgdesc="Package 3"
  depends=("glib2")
  optdepends=("mypackage1: for foobar functionality")
  install=somescript.install
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install3
}
"""

valid_pkgbuilds = [_basic, _split]

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - package variables
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

"These rules checks basic sanity of package metadata"

import re
from Namcap.ruleclass import PkgInfoRule

class CapsPkgnameRule(PkgInfoRule):
	name = "capsnamespkg"
	description = "Verifies package name in package does not include upper case letters"
	def analyze(self, pkginfo, tar):
		if re.search('[A-Z]', pkginfo["name"]) != None:
			self.errors.append(("package-name-in-uppercase", ()))

class UrlRule(PkgInfoRule):
	name = "urlpkg"
	description = "Verifies url is included in a package file"
	def analyze(self, pkginfo, tar):
		if "url" not in pkginfo:
			self.errors.append(("missing-url", ()))

class LicenseRule(PkgInfoRule):
	name = "license"
	description = "Verifies license is included in a PKGBUILD"
	def analyze(self, pkginfo, tar):
		if "license" not in pkginfo or len(pkginfo["license"]) == 0:
			self.errors.append(("missing-license", ()))

# vim: set ts=4 sw=4 noet:



#
# namcap rules - rpath
# Copyright (C) 2009 Abhishek Dasgupta <abhidg@gmail.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os, subprocess, re
import tempfile
from Namcap.util import is_elf, clean_filename
from Namcap.ruleclass import *

allowed = ['/usr/lib', '/usr/lib32', '/lib', '$ORIGIN', '${ORIGIN}']
allowed_toplevels = [s + '/' for s in allowed]
warn = ['/usr/local/lib']
libpath = re.compile('Library rpath: \[(.*)\]')

def get_rpaths(filename):
	p = subprocess.Popen(["readelf", "-d", filename],
		env={'LANG': 'C'},
		stdout=subprocess.PIPE,
		stderr=subprocess.PIPE)
	var = p.communicate()
	if p.returncode != 0:
		raise IOError("unable to read ELF file")

	for j in var[0].decode('ascii').splitlines():
		n = libpath.search(j)
		# Is this a Library rpath: line?
		if n is None:
			continue

		if ":" in n.group(1):
			rpaths = n.group(1).split(':')
		else:
			rpaths = [n.group(1)]
		for path in rpaths:
			yield path

class package(TarballRule):
	name = "rpath"
	description = "Verifies correct and secure RPATH for files."
	def analyze(self, pkginfo, tar):
		for entry in tar:
			if not entry.isfile():
				continue

			# is it an ELF file ?
			f = tar.extractfile(entry)
			elf = f.read()
			f.close()
			if elf[:4] != b"\x7fELF":
				continue # not an ELF file

			# write it to a temporary file
			f = tempfile.NamedTemporaryFile(delete = False)
			f.write(elf)
			f.close()

			for path in get_rpaths(f.name):
				path_ok = path in allowed
				for allowed_toplevel in allowed_toplevels:
					if path.startswith(allowed_toplevel):
						path_ok = True

				if not path_ok:
					self.errors.append(("insecure-rpath %s %s",
						(path, entry.name)))
					break
				if path in warn and entry.name not in insecure_rpaths:
					self.warnings.append(("insecure-rpath %s %s",
						(path, entry.name)))

			os.unlink(f.name)

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - licensepkg
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import os.path
from Namcap.ruleclass import *

class package(TarballRule):
	name = "licensepkg"
	description = "Verifies license is included in a package file"
	def analyze(self, pkginfo, tar):
		if 'license' not in pkginfo or len(pkginfo["license"]) == 0:
			self.errors.append(("missing-license", ()))
		else:
			licensepaths = [x for x in tar.getnames() if x.startswith('usr/share/licenses') and not x.endswith('/')]
			licensedirs = [os.path.split(os.path.split(x)[0])[1] for x in licensepaths]
			licensefiles = [os.path.split(x)[1] for x in licensepaths]
			# Check all licenses for validity
			for license in pkginfo["license"]:
				lowerlicense = license.lower()
				if lowerlicense.startswith('custom') or lowerlicense in ("bsd", "mit", "isc", "python", "zlib", "libpng"):
					if pkginfo["name"] not in licensedirs:
						self.errors.append(("missing-custom-license-dir usr/share/licenses/%s", pkginfo["name"]))
					elif len(licensefiles) == 0:
						self.errors.append(("missing-custom-license-file usr/share/licenses/%s/*", pkginfo["name"]))
				# A common license
				else:
					commonlicenses = [x.lower() for x in os.listdir('/usr/share/licenses/common')]
					if lowerlicense not in commonlicenses:
						self.errors.append(("not-a-common-license %s", license))

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap tests - pkginfo
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
# 

from Namcap.tests.pkgbuild_test import PkgbuildTest
from Namcap.rules.pkginfo import *

class NamcapInvalidPkgNameTest(PkgbuildTest):
	pkgbuild1 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=MyPackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
license=('GPL')
depends=('glibc')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""

	test_valid = PkgbuildTest.valid_tests

	def preSetUp(self):
		self.rule = CapsPkgnameRule

	def test_example1(self):
		# Example 1
		r = self.run_on_pkg(self.pkgbuild1)
		self.assertEqual(r.errors, [("package-name-in-uppercase", ())])
		self.assertEqual(r.warnings, [])
		self.assertEqual(r.infos, [])

class NamcapLicenseTest(PkgbuildTest):
	pkgbuild1 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
url="http://www.example.com/"
depends=('glibc')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""
	test_valid = PkgbuildTest.valid_tests

	def preSetUp(self):
		self.rule = LicenseRule

	def test_example1(self):
		# Example 1
		r = self.run_on_pkg(self.pkgbuild1)
		self.assertEqual(r.errors, [("missing-license", ())] )
		self.assertEqual(r.warnings, [])
		self.assertEqual(r.infos, [])

class NamcapUrlTest(PkgbuildTest):
	pkgbuild1 = """
# Maintainer: Arch Linux <archlinux@example.com>
# Contributor: Arch Linux <archlinux@example.com>

pkgname=mypackage
pkgver=1.0
pkgrel=1
pkgdesc="A package"
arch=('i686' 'x86_64')
depends=('glibc')
license=('GPL')
options=('!libtool')
source=(ftp://ftp.example.com/pub/mypackage-0.1.tar.gz)
md5sums=('abcdefabcdef12345678901234567890')

build() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make
}

package() {
  cd "${srcdir}"/${pkgname}-${pkgver}
  ./configure --prefix=/usr
  make DESTDIR="${pkgdir}" install
}
"""

	test_valid = PkgbuildTest.valid_tests

	def preSetUp(self):
		self.rule = UrlRule

	def test_example1(self):
		# Example 1
		r = self.run_on_pkg(self.pkgbuild1)
		self.assertEqual(r.errors, [("missing-url", ())] )
		self.assertEqual(r.warnings, [])
		self.assertEqual(r.infos, [])

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - symlink
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
import os
from Namcap.ruleclass import *

class package(TarballRule):
	name = "symlink"
	description = "Checks that symlinks point to the right place"
	def analyze(self, pkginfo, tar):
		filenames = [s.name for s in tar]
		for i in tar:
			if i.issym():
				self.infos.append(("symlink-found %s points to %s", (i.name, i.linkname)))
				linktarget = i.linkname
				linklead = os.path.dirname(i.name)
				while linktarget[:3] == "../":
					linktarget = linktarget[3:]
					linklead = linklead.rpartition("/")[0]
				link = linklead + "/" + linktarget
				if link[0] == "/": link = link[1:]
				if i.linkname[0] == "/": link = i.linkname[1:]
				if link not in filenames:
					self.errors.append(("dangling-symlink %s points to %s", (i.name, i.linkname)))
			if i.islnk():
				self.infos.append(("hardlink-found %s points to %s", (i.name, i.linkname)))
				if i.linkname not in filenames:
					self.errors.append(("dangling-hardlink %s points to %s", (i.name, i.linkname)))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - extravars
# Copyright (C) 2003-2009 Jesse Young <jesseyoung@gmail.com>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import re
from Namcap.ruleclass import *

class package(PkgbuildRule):
	name = "extravars"
	description = "Verifies that extra variables start with an underscore"
	def analyze(self, pkginfo, tar):
		stdvars = ['arch', 'license', 'depends', 'makedepends',
				 'provides', 'conflicts' , 'replaces', 'backup',
				 'source', 'noextract', 'md5sums', 'sha1sums',
				 'sha256sums', 'sha384sums', 'sha512sums', 'pkgname',
				 'pkgbase', 'pkgver', 'pkgrel', 'pkgdesc', 'groups',
				 'url', 'install', 'changelog',
				 'options', 'optdepends']
		for i in pkginfo.pkgbuild:
			m = re.match('[\s]*([a-z][^\s=]*[^+])=', i)
			if m:
				varname = m.group(1)
				if varname not in stdvars:
					if not varname.startswith('_'):
						self.warnings.append(("extra-var-begins-without-underscore %s", varname))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - sfurl
# Copyright (C) 2004-2009 Ben Mazer <ben@benmazer.net>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import re
from Namcap.ruleclass import *

class package(PkgbuildRule):
	name = "sfurl"
	description = "Checks for proper sourceforge URLs"
	def analyze(self, pkginfo, tar):
		if 'source' in pkginfo:
			for source in pkginfo["source"]:
				if re.match('(http://|ftp://)\w+.dl.(sourceforge|sf).net', source) != None:
					self.warnings.append(("specific-sourceforge-mirror", ()))
				if re.match('(http://|ftp://)dl.(sourceforge|sf).net', source) != None:
					self.warnings.append(("using-dl-sourceforge", ()))

# vim: set ts=4 sw=4 noet:



#
# namcap rules - elffiles
# Copyright (C) 2009 Hugo Doria <hugo@archlinux.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
from Namcap.util import is_elf, clean_filename
from Namcap.ruleclass import *

# Valid directories for ELF files
valid_dirs = ['bin/', 'sbin/', 'usr/bin/', 'usr/sbin/', 'lib/',
		'usr/lib/', 'usr/lib32/']

class package(TarballRule):
	name = "elffiles"
	description = "Check about ELF files outside some standard paths."
	def analyze(self, pkginfo, tar):
		invalid_elffiles = []

		for entry in tar:
			# is it a regular file ?
			if not entry.isfile():
				continue
			# is it outside standard binary dirs ?
			is_outside_std_dirs = True
			for d in valid_dirs:
				if entry.name.startswith(d):
					is_outside_std_dirs = False
					break
			if not is_outside_std_dirs:
				continue
			# is it an ELF file ?
			f = tar.extractfile(entry)
			if f.read(4) == b"\x7fELF":
				invalid_elffiles.append(entry.name)

		self.errors = [("elffile-not-in-allowed-dirs %s", i)
				for i in invalid_elffiles]

# vim: set ts=4 sw=4 noet:



#! /usr/bin/env python3
# 
# namcap - A Pacman package analyzer
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# 

# System wide global stuff
import sys, os, os.path, imp, getopt, types, tarfile, re, string, Namcap, pacman
import shutil
import Namcap.depends

sandbox_directory = '/tmp/namcap.' + str(os.getpid())

# Functions
def get_modules():
	"""Return all possible modules (rules)"""
	return Namcap.rules.all_rules

def usage():
	"""Display usage information"""
	print("")
	print("Usage: " + sys.argv[0] + " [OPTIONS] packages")
	print("")
	print("Options are:")
	print("    -L, --list                       : list available rules")
	print("    -i                               : prints information (debug) responses from rules")
	print("    -m                               : makes the output parseable (machine-readable)")
	print("    -e rulelist, --exclude=rulelist  : don't apply RULELIST rules to the package")
	print("    -r rulelist, --rules=rulelist    : only apply RULELIST rules to the package")
	print("    -t tags                          : use a custom tag file")

	sys.exit(2)

def verify_package(filename):
	"""Checks if a tarball is a valid package"""
	if not os.path.isfile(filename):
		return 0
	if not tarfile.is_tarfile(filename):
		return 0
	try:
		tar = tarfile.open(package, "r")
		if not tar:
			return 0		
		if not len(tar.getmembers()) > 0:
			tar.close()
			return 0
		if not '.PKGINFO' in tar.getnames():
			tar.close()
			return 0
	except IOError:
		if tar:
			tar.close()
		return 0
	return tar

def process_tags(filename=None, machine=False):
	if machine:
		return lambda s: s
	tags = {}
	if not filename:
		filename="/usr/share/namcap/namcap-tags"
	f = open(filename)
	for i in f.readlines():
		if i[0] == "#" or i.strip() == "": continue
		tagdata = i[:-1].split("::")
		tags[tagdata[0].strip()] = tagdata[1].strip()

	return lambda s: tags[s]

def check_rules_exclude(optlist):	
	'''Check if the -r (--rules) and the -r (--exclude) options
	are being used at same time'''
	args_used = 0
	for i in optlist:	
		if '-r' in i or '-e' in i:
			args_used += 1
		if '--rules' in i or '--exclude' in i:
			args_used += 1			
	return args_used

def show_messages(name, key, messages):
	for msg in messages:
		print("%s %s: %s" % (name, key, m(msg[0]) % msg[1]))

def process_realpackage(package, modules):
	"""Runs namcap checks over a package tarball"""
	extracted = 0
	pkgtar = verify_package(package)

	if not pkgtar:
		print("Error: %s is empty or is not a valid package" % package)
		return 1

	pkginfo = pacman.load(package)
	pkginfo.detected_deps = []
	# Loop through each one, load them apply if possible
	for i in modules:
		rule = get_modules()[i]()

		if isinstance(rule, Namcap.ruleclass.PkgInfoRule):
			rule.analyze(pkginfo, None)
		elif isinstance(rule, Namcap.ruleclass.PkgdirRule):
			# If it's not extracted, then extract it and then analyze the package
			if not extracted:
				os.mkdir(sandbox_directory)
				for j in pkgtar.getmembers():
					pkgtar.extract(j, sandbox_directory)
				extracted = 1
			rule = pkg.analyze(pkginfo, sandbox_directory)
		elif isinstance(rule, Namcap.ruleclass.PkgbuildRule):
			pass
		elif isinstance(rule, Namcap.ruleclass.TarballRule):
			rule.analyze(pkginfo, pkgtar)
		else:
			show_messages(pkginfo["name"], 'E',
			              [('error-running-rule %s', i)])

		# Output the three types of messages
		show_messages(pkginfo["name"], 'E', rule.errors)
		show_messages(pkginfo["name"], 'W', rule.warnings)
		if info_reporting:
			show_messages(pkginfo["name"], 'I', rule.infos)
	
	# dependency analysis
	errs, warns, infos = Namcap.depends.analyze_depends(pkginfo)
	show_messages(pkginfo["name"], 'E', errs)
	show_messages(pkginfo["name"], 'W', warns)
	if info_reporting:
		show_messages(pkginfo["name"], 'I', infos)

	# Clean up if we extracted anything
	if extracted:
		shutil.rmtree(sandbox_directory)

def process_pkginfo(pkginfo, modules):
	"""Runs namcap checks of a single, non-split PacmanPackage object"""
	for i in modules:
		rule = get_modules()[i]()
		if isinstance(rule, Namcap.ruleclass.PkgInfoRule):
			ret = rule.analyze(pkginfo, None)

		# Output the messages
		if "base" in pkginfo:
			name = "PKGBUILD (" + pkginfo["base"] + ")"
		else:
			name = "PKGBUILD (" + pkginfo["name"] + ")"
		show_messages(name, 'E', rule.errors)
		show_messages(name, 'W', rule.warnings)
		if info_reporting:
			show_messages(name, 'I', rule.infos)


def process_pkgbuild(package, modules):
	"""Runs namcap checks over a PKGBUILD"""
	# We might want to do some verifying in here... but really... isn't that
	# what pacman.load is for?
	pkginfo = pacman.load(package)

	if pkginfo == None:
		print("Error: %s is not a valid PKGBUILD" % package)
		return 1

	# apply global PKGBUILD rules
	for i in modules:
		rule = get_modules()[i]()
		if isinstance(rule, Namcap.ruleclass.PkgbuildRule):
			ret = rule.analyze(pkginfo, package)
		# Output the messages
		if "base" in pkginfo:
			name = "PKGBUILD (" + pkginfo["base"] + ")"
		else:
			name = "PKGBUILD (" + pkginfo["name"] + ")"
		show_messages(name, 'E', rule.errors)
		show_messages(name, 'W', rule.warnings)
		if info_reporting:
			show_messages(name, 'I', rule.infos)
	# apply per pkginfo rule
	for subpkg in (pkginfo.subpackages
			if pkginfo.is_split else [pkginfo]):
		process_pkginfo(subpkg, modules)

# Main
modules = get_modules()
info_reporting = 0
machine_readable = False
filename = None

# get our options and process them
try:
	optlist, args = getopt.getopt(sys.argv[1:], "ihmr:e:t:L",
			["info", "help", "machine-readable", "rules=",
				"exclude=", "tags=", "list"])
except getopt.GetoptError:
	usage()

active_modules = {}

# Verifying if we are using the -r and -r options at same time
if check_rules_exclude(optlist) > 1:
	print("You cannot use '-r' (--rules) and '-e' (-exclude) options at same time")
	usage()

for i, k in optlist:
	if i in ('-L', '--list'):
		print("-"*20 + " Namcap rule list " + "-"*20)
		for j in sorted(modules):
			print("%-20s: %s" % (j, modules[j].description))
		sys.exit(2)

	if i in ('-r', '--rules'):
		module_list = k.split(',')
		for j in module_list:
			if j in modules:
				active_modules[j] = modules[j]
			else:
				print("Error: Rule '%s' does not exist" % j)
				usage()

	# Used to exclude some rules from the check
	if i in ('-e', '--exclude'):
		module_list = k.split(',')
		active_modules.update(modules)
		for j in module_list:
			if j in modules:
				active_modules.pop(j)
			else:
				print("Error: Rule '%s' does not exist" % j)
				usage()

	if i in ('-i', '--info'):
		info_reporting = 1

	if i in ('-h', '--help'):
		usage()
	if i in ('-m', '--machine-readable'):
		machine_readable = True

	if i in ('-t', '--tags'):
		filename = k

# If there are no args, print usage
if (args == []):
	usage()

m = process_tags(filename=filename, machine=machine_readable)
packages = args

# No rules selected?  Then select them all!
if len(active_modules) == 0:
	active_modules = modules

# Go through each package, get the info, and apply the rules
for package in packages:
	if not os.access(package, os.R_OK):
		print("Error: Problem reading %s" % package)
		usage()

	if os.path.isfile(package) and tarfile.is_tarfile(package):
		process_realpackage(package, active_modules)
	elif package.endswith('PKGBUILD'):
		process_pkgbuild(package, active_modules)
	else:
		print("Error: Cannot process %s" % package)

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - perllocal
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

from Namcap.ruleclass import *

class package(TarballRule):
	name = "perllocal"
	description = "Verifies the absence of perllocal.pod."
	def analyze(self, pkginfo, tar):
		j = 'perllocal.pod'
		for i in tar.getnames():
			if i[-len(j):] == j:
				self.errors.append(("perllocal-pod-present %s", i))

# vim: set ts=4 sw=4 noet:



#
# namcap rules - infodirectory
# Copyright (C) 2008-2009 Allan McRae <allan@archlinux.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from Namcap.ruleclass import *

class InfodirRule(TarballRule):
	name = "infodirectory"
	description = "Checks for info directory file."
	def analyze(self, pkginfo, tar):
		for i in tar.getnames():
			if i == "usr/share/info/dir":
				self.errors.append(("info-dir-file-present %s", i))

class InfoInstallRule(TarballRule):
	name = "infoinstall"
	description = "Checks that info files are correctly installed."
	def analyze(self, pkginfo, tar):
		info_installed = False
		info_present = False

		for i in tar:
			if i.name == ".INSTALL":
				install = tar.extractfile(i)
				if b"install-info" in install.read():
					info_installed = True
				install.close()

			if i.name.startswith("usr/share/info/"):
				info_present = True

		if info_present and not info_installed:
			self.errors.append(("info-dir-not-updated", ()))

# vim: set ts=4 sw=4 noet:



#
# namcap rules - utility functions
# Copyright (C) 2009 Dan McGee <dan@archlinux.org>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os
import re
import stat

def read_carefully(path, readcall):
	if not os.path.isfile(path):
		return False
	reset_perms = False
	if not os.access(path, os.R_OK):
		# don't mess with links we can't read
		if os.path.islink(path):
			return None
		reset_perms = True
		# attempt to make it readable if possible
		statinfo = os.stat(path)
		newmode = statinfo.st_mode | stat.S_IRUSR
		try:
			os.chmod(path, newmode)
		except IOError:
			return None
	fd = open(path, 'rb')
	val = readcall(fd)
	fd.close()
	# reset permissions if necessary
	if reset_perms:
		# set file back to original permissions
		os.chmod(path, statinfo.st_mode)
	return val

def is_elf(path):
	"""
	Given a file path, ensure it exists and peek at the first few bytes
	to determine if it is an ELF file.
	"""
	bytes = read_carefully(path, lambda fd: fd.read(4))
	if not bytes:
		return False
	# magic elf header, present in binaries and libraries
	if bytes == b"\x7FELF":
		return True
	else:
		return False

def script_type(path):
	firstline = read_carefully(path, lambda fd: fd.readline())
	if not firstline:
		return None
	script = re.compile(b'#!.*/(.*)')
	m = script.match(firstline)
	if m != None:
		cmd = m.group(1).split()
		name = cmd[0]
		if name == 'env':
			name = cmd[1]
		return name.decode('ascii')
	return None

clean_filename = lambda s: re.search(r"/tmp/namcap\.[0-9]*/(.*)", s).group(1)

# vim: set ts=4 sw=4 noet:



#!/usr/bin/env python
from setuptools import setup, find_packages

DATAFILES = [('/usr/share/man/man1', ['namcap.1']),
		('/usr/share/namcap', ['namcap-tags', 'parsepkgbuild.sh']),
		('/usr/share/doc/namcap',['README','AUTHORS','TODO'])]

setup(name="namcap",
	version="3.0.1",

	description="Pacman package analyzer",
	author="Arch Dev Team",
	author_email="pacman-dev@archlinux.org",
	url="http://www.archlinux.org/",

	py_modules=["namcap", "pacman"],
	packages = find_packages(),
	scripts=["namcap", 'parsepkgbuild'],
	test_suite = "Namcap.tests",
	data_files=DATAFILES)

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap rules - kdeprograms
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
# 

from Namcap.ruleclass import *
import Namcap.depends
class package(TarballRule):
	name = "kdeprograms"
	description = "Checks that KDE programs have kdebase-runtime as a dependency"
	def analyze(self, pkginfo, tar):
		binaries = [ f.name for f in tar
				if f.name.startswith("usr/bin") and f.isfile() ]
		if len(binaries) == 0:
			return
		# If there are binaries, check for a kdelibs dependency
		if "depends" in pkginfo:
			if 'kdelibs' in pkginfo["depends"] or 'kdelibs' in Namcap.depends.getcovered(pkginfo["depends"]):
				self.infos.append(("kdebase-runtime-needed-dep %s", binaries))
				pkginfo.detected_deps.append("kdebase-runtime")

# vim: set ts=4 sw=4 noet:



# -*- coding: utf-8 -*-
#
# namcap rules - sodepends
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

"""Checks dependencies resulting from linking of shared libraries."""

import re, os, pacman
import subprocess
import tempfile
from Namcap.ruleclass import *

libcache = {'i686': {}, 'x86-64': {}}

def figurebitsize(line):
	"""
	Given a line of output from readelf (usually Shared library:) return
	'i686' or 'x86-64' if the binary is a 32bit or 64bit binary
	"""

	address = line.split()[0]
	if len(address) == 18: # + '0x' + 16 digits
		return 'x86-64'
	else:
		return 'i686'

def scanlibs(fileobj, filename, sharedlibs):
	"""
	Run "readelf -d" on a file-like object (e.g. a TarFile)
	If it depends on a library, store that library's path.
	"""
	shared = re.compile('Shared library: \[(.*)\]')

	# test magic bytes
	magic = fileobj.read(4)
	if magic[:4] != b"\x7fELF":
		return

	# read the rest of file
	tmp = tempfile.NamedTemporaryFile(delete=False)
	tmp.write(magic + fileobj.read())
	tmp.close()

	try:
		p = subprocess.Popen(["readelf", "-d", tmp.name],
				env = {"LANG": "C"},
				stdout=subprocess.PIPE,
				stderr=subprocess.PIPE)
		var = p.communicate()
		assert(p.returncode == 0)
		for j in var[0].decode('ascii').splitlines():
			n = shared.search(j)
			# Is this a Shared library: line?
			if n != None:
				# Find out its architecture
				architecture = figurebitsize(j)
				try:
					libpath = os.path.abspath(
							libcache[architecture][n.group(1)])[1:]
					sharedlibs.setdefault(libpath, {})[filename] = 1
				except KeyError:
					# We didn't know about the library, so add it for fail later
					sharedlibs.setdefault(n.group(1), {})[filename] = 1
	finally:
		os.unlink(tmp.name)

def finddepends(liblist):
	dependlist = {}

	somatches = {}
	actualpath = {}

	knownlibs = set(liblist.keys())
	foundlibs = set()

	for j in knownlibs:
		actualpath[j] = os.path.realpath('/'+j)[1:]

	# Sometimes packages don't include all so .so, .so.1, .so.1.13, .so.1.13.19 files
	# They rely on ldconfig to create all the symlinks
	# So we will strip off the matching part of the files and use this regexp to match the rest
	so_end = re.compile('(\.\d+)*')
	# Whether we should even look at a particular file
	is_so = re.compile('\.so')

	pacmandb = '/var/lib/pacman/local'
	for i in os.listdir(pacmandb):
		if os.path.isfile(pacmandb+'/'+i+'/files'):
			file = open(pacmandb+'/'+i+'/files', errors='ignore')
			for j in file:
				if not is_so.search(j):
					continue

				for k in knownlibs:
					# File must be an exact match or have the right .so ending numbers
					# i.e. gpm includes libgpm.so and libgpm.so.1.19.0, but everything links to libgpm.so.1
					# We compare find libgpm.so.1.19.0 startswith libgpm.so.1 and .19.0 matches the regexp
					if j == actualpath[k] or (j.startswith(actualpath[k]) and so_end.match(j[len(actualpath[k]):])):
						n = re.match('(.*)-([^-]*)-([^-]*)', i)
						if n.group(1) not in dependlist:
							dependlist[n.group(1)] = {}
						for x in liblist[k]:
							dependlist[n.group(1)][x] = 1
						foundlibs.add(k)
			file.close()

	orphans = list(knownlibs - foundlibs)
	return dependlist, orphans

def filllibcache():
	var = subprocess.Popen('ldconfig -p', 
			shell=True,
			stdout=subprocess.PIPE,
			stderr=subprocess.PIPE).communicate()
	libline = re.compile('\s*(.*) \((.*)\) => (.*)')
	for j in var[0].decode('ascii').splitlines():
		g = libline.match(j)
		if g != None:
			if g.group(2).startswith('libc6,x86-64'):
				libcache['x86-64'][g.group(1)] = g.group(3)
			else:
				libcache['i686'][g.group(1)] = g.group(3)


class SharedLibsRule(TarballRule):
	name = "sodepends"
	description = "Checks dependencies caused by linked shared libraries"
	def analyze(self, pkginfo, tar):
		liblist = {}
		dependlist = {}
		filllibcache()
		os.environ['LC_ALL'] = 'C'

		for entry in tar:
			if not entry.isfile():
				continue
			f = tar.extractfile(entry)
			scanlibs(f, entry.name, liblist)
			f.close()

		# Ldd all the files and find all the link and script dependencies
		dependlist, orphans = finddepends(liblist)

		# Handle "no package associated" errors
		self.warnings.extend([("library-no-package-associated %s", i)
			for i in orphans])

		# Print link-level deps
		for i, v in dependlist.items():
			if type(v) == dict:
				files = list(v.keys())
				pkginfo.detected_deps.append(i)
				self.infos.append(("link-level-dependence %s in %s", (i, str(files))))

		# Check for packages in testing
		if os.path.isdir('/var/lib/pacman/sync/testing'):
			for i in dependlist.keys():
				p = pacman.load(i, '/var/lib/pacman/sync/testing/')
				q = pacman.load(i)
				if p != None and q != None and p["version"] == q["version"]:
					self.warnings.append(("dependency-is-testing-release %s", i))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - fhs
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import os
import tarfile
from Namcap.ruleclass import *

class FHSRule(TarballRule):
	name = "directoryname"
	description = "Checks for standard directories."
	def analyze(self, pkginfo, tar):
		valid_paths = [
				'bin/', 'etc/', 'lib/', 'sbin/', 'opt/', 'srv/',
				'usr/bin/', 'usr/include/', 'usr/lib/', 'usr/lib32/',
				'usr/sbin/', 'usr/share/',
				'var/cache/', 'var/lib/', 'var/lock/', 'var/log/', 'var/opt/',
				'var/run/', 'var/spool/', 'var/state/',
				'.PKGINFO', '.INSTALL', '.CHANGELOG',
		]
		for i in tar.getnames():
			# Replace multiple /'s at the end of a string with a single /
			# Not sure if this is a python bug or a makepkg bug
			if i.endswith('/'):
				i = i.rstrip('/') + '/'
			fileok = 0
			for j in valid_paths:
				# matches directory names (j, filename) or parent directories (filename, j)
				if i[0:len(j)] == j or j[0:len(i)] == i:
					fileok = 1
			if not fileok:
				self.warnings.append(("file-in-non-standard-dir %s", i))

class FHSManpagesRule(TarballRule):
	name = "fhs-manpages"
	description = "Verifies correct installation of man pages"
	def analyze(self, pkginfo, tar):
		gooddir = 'usr/share/man'
		bad_dir = 'usr/man'
		for i in tar.getmembers():
			if not i.isfile():
				continue
			if i.name.startswith(bad_dir):
				self.errors.append(("non-fhs-man-page %s", i.name))
			elif not i.name.startswith(gooddir):
				#Check everything else to see if it has a 'man' path component
				for part in i.name.split(os.sep):
					if part == "man":
						self.warnings.append(("potential-non-fhs-man-page %s", i.name))

class FHSInfoPagesRule(TarballRule):
	name = "fhs-infopages"
	description = "Verifies correct installation of info pages"
	def analyze(self, pkginfo, tar):
		for i in tar.getmembers():
			if not i.isfile():
				continue
			if i.name.startswith('usr/info'):
				self.errors.append(("non-fhs-info-page %s", i.name))
			elif not i.name.startswith('usr/share/info'):
				for part in i.name.split(os.sep):
					if part == "info":
						self.warnings.append(("potential-non-fhs-info-page %s", i.name))

# vim: set ts=4 sw=4 noet:



# 
# namcap rules - gnomemime
# Copyright (C) 2003-2009 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
# 

from Namcap.ruleclass import *

class package(TarballRule):
    name = "gnomemime"
    description = "Checks for generated GNOME mime files"
    def analyze(self, pkginfo, tar):
        mime_files = [
                'usr/share/applications/mimeinfo.cache',
                'usr/share/mime/XMLnamespaces', 
                'usr/share/mime/aliases', 
                'usr/share/mime/globs', 
                'usr/share/mime/magic', 
                'usr/share/mime/subclasses'
                ]

        for i in tar.getnames():
            if i in mime_files:
                self.errors.append(("gnome-mime-file %s", i))
                
# vim: set ts=4 sw=4 noet:



# 
# namcap rules - glibfiles
# Copyright (C) 2011 Rémy Oudompheng <remy@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
# 

from Namcap.ruleclass import *

class GlibSchemasRule(TarballRule):
	name = "glibschemas"
	description = "Check that dconf schemas are compiled"
	def analyze(self, pkginfo, tar):
		flag = False
		ok = False
		for entry in tar:
			if 'usr/share/glib-2.0/schemas' in entry.name and not flag:
				flag = True
				pkginfo.detected_deps.append("dconf")
			if ".INSTALL" in entry.name:
				f = tar.extractfile(".INSTALL")
				if b"glib-compile-schemas" in f.read():
					ok = True
				f.close()
		if flag and not ok:
			self.warnings.append(("dconf-schemas-not-compiled", ()))

class GioModulesRule(TarballRule):
	name = "giomodules"
	description = "Check that GIO modules are registered"
	def analyze(self, pkginfo, tar):
		flag = False
		ok = False
		for entry in tar:
			if 'usr/lib/gio/modules' in entry.name and not flag:
				flag = True
				pkginfo.detected_deps.append("glib2")
			if ".INSTALL" in entry.name:
				f = tar.extractfile(".INSTALL")
				if b"gio-querymodules" in f.read():
					ok = True
				f.close()
		if flag and not ok:
			self.warnings.append(("gio-modules-not-registered", ()))

# vim: set ts=4 sw=4 noet:
